@@ -5,13 +5,13 @@ import json
 from http import HTTPStatus
 from typing import Iterable, Mapping, Union
 
+from amundsen_common.entity.resource_type import ResourceType
 from amundsen_common.models.lineage import LineageSchema
 from flasgger import swag_from
 from flask import request
 from flask_restful import Resource, reqparse
 
 from metadata_service.api.badge import BadgeCommon
-from metadata_service.entity.resource_type import ResourceType
 from metadata_service.exception import NotFoundException
 from metadata_service.proxy import get_proxy_client
 
