@@ -4,6 +4,7 @@
 from abc import ABCMeta, abstractmethod
 from typing import Any, Dict, List, Optional, Tuple, Union
 
+from amundsen_common.entity.resource_type import ResourceType
 from amundsen_common.models.dashboard import DashboardSummary
 from amundsen_common.models.feature import Feature
 from amundsen_common.models.generation_code import GenerationCode
@@ -16,7 +17,6 @@ from flask import current_app as app
 from metadata_service.entity.dashboard_detail import \
     DashboardDetail as DashboardDetailEntity
 from metadata_service.entity.description import Description
-from metadata_service.entity.resource_type import ResourceType
 from metadata_service.util import UserResourceRel
 
 
@@ -121,6 +121,13 @@ class BaseProxy(metaclass=ABCMeta):
                            user_id: Optional[str] = None) -> List[PopularTable]:
         pass
 
+    @abstractmethod
+    def get_popular_resources(self, *,
+                              num_entries: int,
+                              resource_types: List[str],
+                              user_id: Optional[str] = None) -> Dict[str, List]:
+        raise NotImplementedError
+
     @abstractmethod
     def get_latest_updated_ts(self) -> int:
         pass
