@@ -9,6 +9,7 @@ from typing import (Any, Dict, Iterable, List, Optional, Tuple,  # noqa: F401
                     Union, no_type_check)
 
 import neo4j
+from amundsen_common.entity.resource_type import ResourceType, to_resource_type
 from amundsen_common.models.dashboard import DashboardSummary
 from amundsen_common.models.feature import Feature, FeatureWatermark
 from amundsen_common.models.generation_code import GenerationCode
@@ -32,7 +33,6 @@ from metadata_service.entity.dashboard_detail import \
 from metadata_service.entity.dashboard_query import \
     DashboardQuery as DashboardQueryEntity
 from metadata_service.entity.description import Description
-from metadata_service.entity.resource_type import ResourceType
 from metadata_service.entity.tag_detail import TagDetail
 from metadata_service.exception import NotFoundException
 from metadata_service.proxy.base_proxy import BaseProxy
@@ -42,7 +42,7 @@ from metadata_service.util import UserResourceRel
 _CACHE = CacheManager(**parse_cache_config_options({'cache.type': 'memory'}))
 
 # Expire cache every 11 hours + jitter
-_GET_POPULAR_TABLE_CACHE_EXPIRY_SEC = 11 * 60 * 60 + randint(0, 3600)
+_GET_POPULAR_RESOURCES_CACHE_EXPIRY_SEC = 11 * 60 * 60 + randint(0, 3600)
 
 
 CREATED_EPOCH_MS = 'publisher_created_epoch_ms'
@@ -197,7 +197,7 @@ class Neo4jProxy(BaseProxy):
         readers = []  # type: List[Reader]
         for usage_neo4j_record in usage_neo4j_records:
             reader_data = self._get_user_details(user_id=usage_neo4j_record['email'])
-            reader = Reader(user=User(**reader_data),
+            reader = Reader(user=self._build_user_from_record(record=reader_data),
                             read_count=usage_neo4j_record['read_count'])
             readers.append(reader)
 
@@ -278,7 +278,7 @@ class Neo4jProxy(BaseProxy):
 
         for owner in table_records.get('owner_records', []):
             owner_data = self._get_user_details(user_id=owner['email'])
-            owner_record.append(User(**owner_data))
+            owner_record.append(self._build_user_from_record(record=owner_data))
 
         src = None
 
@@ -1004,8 +1004,9 @@ class Neo4jProxy(BaseProxy):
             return neo4j_statistics
         return {}
 
-    @_CACHE.cache('_get_global_popular_tables_uris', expire=_GET_POPULAR_TABLE_CACHE_EXPIRY_SEC)
-    def _get_global_popular_tables_uris(self, num_entries: int) -> List[str]:
+    @_CACHE.cache('_get_global_popular_resources_uris', expire=_GET_POPULAR_RESOURCES_CACHE_EXPIRY_SEC)
+    def _get_global_popular_resources_uris(self, num_entries: int,
+                                           resource_type: ResourceType = ResourceType.Table) -> List[str]:
         """
         Retrieve popular table uris. Will provide tables with top x popularity score.
         Popularity score = number of distinct readers * log(total number of reads)
@@ -1017,28 +1018,29 @@ class Neo4jProxy(BaseProxy):
         :return: Iterable of table uri
         """
         query = textwrap.dedent("""
-        MATCH (tbl:Table)-[r:READ_BY]->(u:User)
-        WITH tbl.key as table_key, count(distinct u) as readers, sum(r.read_count) as total_reads
+        MATCH (resource:{resource_type})-[r:READ_BY]->(u:User)
+        WITH resource.key as resource_key, count(distinct u) as readers, sum(r.read_count) as total_reads
         WHERE readers >= $num_readers
-        RETURN table_key, readers, total_reads, (readers * log(total_reads)) as score
+        RETURN resource_key, readers, total_reads, (readers * log(total_reads)) as score
         ORDER BY score DESC LIMIT $num_entries;
-        """)
+        """).format(resource_type=resource_type.name)
         LOGGER.info('Querying popular tables URIs')
-        num_readers = current_app.config['POPULAR_TABLE_MINIMUM_READER_COUNT']
+        num_readers = current_app.config['POPULAR_RESOURCES_MINIMUM_READER_COUNT']
         records = self._execute_cypher_query(statement=query,
                                              param_dict={'num_readers': num_readers,
                                                          'num_entries': num_entries})
 
-        return [record['table_key'] for record in records]
+        return [record['resource_key'] for record in records]
 
     @timer_with_counter
-    @_CACHE.cache('_get_personal_popular_tables_uris', _GET_POPULAR_TABLE_CACHE_EXPIRY_SEC)
-    def _get_personal_popular_tables_uris(self, num_entries: int,
-                                          user_id: str) -> List[str]:
+    @_CACHE.cache('_get_personal_popular_tables_uris', _GET_POPULAR_RESOURCES_CACHE_EXPIRY_SEC)
+    def _get_personal_popular_resources_uris(self, num_entries: int,
+                                             user_id: str,
+                                             resource_type: ResourceType = ResourceType.Table) -> List[str]:
         """
-        Retrieve personalized popular table uris. Will provide tables with top
+        Retrieve personalized popular resources uris. Will provide resources with top
         popularity score that have been read by a peer of the user_id provided.
-        The popularity score is defined in the same way as `_get_global_popular_tables_uris`
+        The popularity score is defined in the same way as `_get_global_popular_resources_uris`
 
         The result of this method will be cached based on the key (num_entries, user_id),
         and the cache will be expired based on _GET_POPULAR_TABLE_CACHE_EXPIRY_SEC
@@ -1046,28 +1048,29 @@ class Neo4jProxy(BaseProxy):
         :return: Iterable of table uri
         """
         statement = textwrap.dedent("""
-        MATCH (:User {key:$user_id})<-[:READ_BY]-(:Table)-[:READ_BY]->
-             (coUser:User)<-[coRead:READ_BY]-(table:Table)
-        WITH table.key AS table_key, count(DISTINCT coUser) AS co_readers,
+        MATCH (:User {{key:$user_id}})<-[:READ_BY]-(:{resource_type})-[:READ_BY]->
+             (coUser:User)<-[coRead:READ_BY]-(resource:{resource_type})
+        WITH resource.key AS resource_key, count(DISTINCT coUser) AS co_readers,
              sum(coRead.read_count) AS total_co_reads
         WHERE co_readers >= $num_readers
-        RETURN table_key, (co_readers * log(total_co_reads)) AS score
+        RETURN resource_key, (co_readers * log(total_co_reads)) AS score
         ORDER BY score DESC LIMIT $num_entries;
-        """)
+        """).format(resource_type=resource_type.name)
         LOGGER.info('Querying popular tables URIs')
-        num_readers = current_app.config['POPULAR_TABLE_MINIMUM_READER_COUNT']
+        num_readers = current_app.config['POPULAR_RESOURCES_MINIMUM_READER_COUNT']
         records = self._execute_cypher_query(statement=statement,
                                              param_dict={'user_id': user_id,
                                                          'num_readers': num_readers,
                                                          'num_entries': num_entries})
 
-        return [record['table_key'] for record in records]
+        return [record['resource_key'] for record in records]
 
     @timer_with_counter
     def get_popular_tables(self, *,
                            num_entries: int,
                            user_id: Optional[str] = None) -> List[PopularTable]:
         """
+
         Retrieve popular tables. As popular table computation requires full scan of table and user relationship,
         it will utilize cached method _get_popular_tables_uris.
 
@@ -1076,10 +1079,10 @@ class Neo4jProxy(BaseProxy):
         """
         if user_id is None:
             # Get global popular table URIs
-            table_uris = self._get_global_popular_tables_uris(num_entries)
+            table_uris = self._get_global_popular_resources_uris(num_entries)
         else:
             # Get personalized popular table URIs
-            table_uris = self._get_personal_popular_tables_uris(num_entries, user_id)
+            table_uris = self._get_personal_popular_resources_uris(num_entries, user_id)
 
         if not table_uris:
             return []
@@ -1106,6 +1109,101 @@ class Neo4jProxy(BaseProxy):
             popular_tables.append(popular_table)
         return popular_tables
 
+    def _get_popular_tables(self, *, resource_uris: List[str]) -> List[TableSummary]:
+        """
+
+        """
+        if not resource_uris:
+            return []
+
+        query = textwrap.dedent("""
+        MATCH (db:Database)-[:CLUSTER]->(clstr:Cluster)-[:SCHEMA]->(schema:Schema)-[:TABLE]->(tbl:Table)
+        WHERE tbl.key IN $table_uris
+        WITH db.name as database_name, clstr.name as cluster_name, schema.name as schema_name, tbl
+        OPTIONAL MATCH (tbl)-[:DESCRIPTION]->(dscrpt:Description)
+        RETURN database_name, cluster_name, schema_name, tbl.name as table_name,
+        dscrpt.description as table_description;
+        """)
+        records = self._execute_cypher_query(statement=query,
+                                             param_dict={'table_uris': resource_uris})
+
+        popular_tables = []
+        for record in records:
+            popular_table = TableSummary(database=record['database_name'],
+                                         cluster=record['cluster_name'],
+                                         schema=record['schema_name'],
+                                         name=record['table_name'],
+                                         description=self._safe_get(record, 'table_description'))
+            popular_tables.append(popular_table)
+        return popular_tables
+
+    def _get_popular_dashboards(self, *, resource_uris: List[str]) -> List[DashboardSummary]:
+        """
+
+        """
+        if not resource_uris:
+            return []
+
+        query = textwrap.dedent(f"""
+        MATCH (d:Dashboard)-[:DASHBOARD_OF]->(dg:Dashboardgroup)-[:DASHBOARD_GROUP_OF]->(c:Cluster)
+        WHERE d.key IN $dashboards_uris
+        OPTIONAL MATCH (d)-[:DESCRIPTION]->(dscrpt:Description)
+        OPTIONAL MATCH (d)-[:EXECUTED]->(last_exec:Execution)
+        WHERE split(last_exec.key, '/')[5] = '_last_successful_execution'
+        RETURN c.name as cluster_name, dg.name as dg_name, dg.dashboard_group_url as dg_url,
+        d.key as uri, d.name as name, d.dashboard_url as url,
+        split(d.key, '_')[0] as product,
+        dscrpt.description as description, last_exec.timestamp as last_successful_run_timestamp""")
+
+        records = self._execute_cypher_query(statement=query,
+                                             param_dict={'dashboards_uris': resource_uris})
+
+        popular_dashboards = []
+        for record in records:
+            popular_dashboards.append(DashboardSummary(
+                uri=record['uri'],
+                cluster=record['cluster_name'],
+                group_name=record['dg_name'],
+                group_url=record['dg_url'],
+                product=record['product'],
+                name=record['name'],
+                url=record['url'],
+                description=record['description'],
+                last_successful_run_timestamp=record['last_successful_run_timestamp'],
+            ))
+
+        return popular_dashboards
+
+    @timer_with_counter
+    def get_popular_resources(self, *,
+                              num_entries: int,
+                              resource_types: List[str],
+                              user_id: Optional[str] = None) -> Dict[str, List]:
+        popular_resources: Dict[str, List] = dict()
+        for resource in resource_types:
+            resource_type = to_resource_type(label=resource)
+            popular_resources[resource_type.name] = list()
+            if user_id is None:
+                # Get global popular Table/Dashboard URIs
+                resource_uris = self._get_global_popular_resources_uris(num_entries,
+                                                                        resource_type=resource_type)
+            else:
+                # Get personalized popular Table/Dashboard URIs
+                resource_uris = self._get_personal_popular_resources_uris(num_entries,
+                                                                          user_id,
+                                                                          resource_type=resource_type)
+
+            if resource_type == ResourceType.Table:
+                popular_resources[resource_type.name] = self._get_popular_tables(
+                    resource_uris=resource_uris
+                )
+            elif resource_type == ResourceType.Dashboard:
+                popular_resources[resource_type.name] = self._get_popular_dashboards(
+                    resource_uris=resource_uris
+                )
+
+        return popular_resources
+
     @timer_with_counter
     def get_user(self, *, id: str) -> Union[UserEntity, None]:
         """
@@ -1204,7 +1302,7 @@ class Neo4jProxy(BaseProxy):
         return [self._build_user_from_record(record=rec) for rec in result['users']]
 
     @staticmethod
-    def _build_user_from_record(record: dict, manager_name: str = '') -> UserEntity:
+    def _build_user_from_record(record: dict, manager_name: Optional[str] = None) -> UserEntity:
         """
         Builds user record from Cypher query result. Other than the one defined in amundsen_common.models.user.User,
         you could add more fields from User node into the User model by specifying keys in config.USER_OTHER_KEYS
@@ -1219,10 +1317,11 @@ class Neo4jProxy(BaseProxy):
                     other_key_values[k] = record[k]
 
         return UserEntity(email=record['email'],
+                          user_id=record.get('user_id', record['email']),
                           first_name=record.get('first_name'),
                           last_name=record.get('last_name'),
                           full_name=record.get('full_name'),
-                          is_active=record.get('is_active', False),
+                          is_active=record.get('is_active', True),
                           github_username=record.get('github_username'),
                           team_name=record.get('team_name'),
                           slack_id=record.get('slack_id'),
@@ -1541,7 +1640,7 @@ class Neo4jProxy(BaseProxy):
 
         for owner in dashboard_record.get('owners', []):
             owner_data = self._get_user_details(user_id=owner['email'], user_data=owner)
-            owners.append(User(**owner_data))
+            owners.append(self._build_user_from_record(record=owner_data))
 
         tags = [Tag(tag_type=tag['tag_type'], tag_name=tag['key']) for tag in dashboard_record['tags']]
 
